/*
Copyright (c) 2011, Julian Lyndon-Smith (julian+maia@dotr.com)
http://www.dotr.com
All rights reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

using Progress.Lang.*.
using dotr.Maia.Model.*.

/** Maia enum builder 
 * extends the basic builder to add methods to build enum files from the template provided
 * @author Julian Lyndon-Smith (julian+maia@dotr.com) 
 */

class dotr.Maia.Builder.BuildEnum inherits dotr.Maia.Builder.Basic: 
  constructor BuildEnum (p_ObjectConfig as ObjectConfig):
    def var ObjectBuilder as dotr.Maia.Builder.ObjectBuilder no-undo.
    
    super(p_ObjectConfig).
    
    assign ObjectBuilder = dotr.Maia.Builder.ObjectBuilder:Instance.
    
    assign BuildPackage   = substitute("&1.Enum",dotr.Maia.CurrentProject:Name)
           BuildDirectory = substitute("&1/Enum",dotr.Maia.CurrentProject:Path).
  end constructor.

  /** build list of static enum options
   * @param p_Directive : holds the directive name and all parameters
   */
  
  method public void enumProperties(p_Directive as Directive):
    def var lv_Data   as char no-undo.
    def var lv_Assign as char no-undo.
    
    def var lv_maxLen as int no-undo.
    
    def var Property1 as Property no-undo.
    
    /** get max size of property name. Used to align code properly */
    for each TTProperty no-lock :
      Property1 = cast(TTProperty.Property,Property).
      if Property1:BuildProperty then assign lv_MaxLen = MAX(lv_MaxLen,length(Property1:FieldName)).
    end.
    
    /** now let's loop through, building the assign statement */
    for each TTProperty no-lock:
      Property1 = cast(TTProperty.Property,Property).
      
      if not Property1:BuildProperty then next.

      assign lv_Assign = substitute("def public static property &1@ as &2 init &3&1&3 no-undo get . private set.~n",
                                    Property1:PropertyName,
                                    Property1:DataType,
                                    if Property1:DataType eq "character" then '"' else ''
                                    )
             lv_Data   = lv_Data + replace(lv_Assign,"@",fill(" ",lv_maxLen - length(Property1:PropertyName))). 
    end.
    
    assign p_Directive:Data = trim(lv_Data,"~n").                                                                                                                                                  
  end method.

  /** build array of enum options
   * @param p_Directive : holds the directive name and all parameters
   */
  
  method public void enumArray(p_Directive as Directive):
    def var lv_Assign as char no-undo.
    
    def var lv_Num as int no-undo.
    
    def var Property1 as Property no-undo.
    
    /** get number of properties */
    for each TTProperty no-lock :
      Property1 = cast(TTProperty.Property,Property).
      if Property1:BuildProperty then assign lv_num = lv_num + 1.
    end.
    
    assign lv_Assign = substitute("extent(Items) = &1.~n~nassign ",lv_num)
           lv_Num    = 0.
    
    /** loop through all properties, building the assign statement */
    for each TTProperty no-lock:
      Property1 = cast(TTProperty.Property,Property).
      
      if not Property1:BuildProperty then next.

      assign lv_num    = lv_num + 1
             lv_Assign = lv_Assign + substitute("Items[&1] = <<BuildPackage>>.<<ClassName>>:&2~n       ",
                                                lv_num,
                                                Property1:PropertyName).
    end.
    
    assign p_Directive:Data = trim(lv_Assign,"~n") + ".".
                                                                                                                                                            
  end method.

end class.