/*
Copyright (c) 2011, Julian Lyndon-Smith (julian+maia@dotr.com)
http://www.dotr.com
All rights reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

using Progress.Lang.*.
using dotr.Maia.Model.*.

/** Maia ribbon builder 
 * extends the basic builder to add methods to build a ribbon class from the BaseRibbon.cls template provided
 * @author Julian Lyndon-Smith (julian+maia@dotr.com) 
 */

class dotr.Maia.Builder.BuildRibbon inherits dotr.Maia.Builder.Basic:
  def var numFields as int no-undo.

  /** create directory structure if required
   */
        
  constructor BuildRibbon (p_ObjectConfig as ObjectConfig):
    def var ObjectBuilder as dotr.Maia.Builder.ObjectBuilder no-undo.
    
    super(p_ObjectConfig).
    
    assign ObjectBuilder = dotr.Maia.Builder.ObjectBuilder:Instance.
    
    assign BuildPackage   = substitute("&1.UI.winform.Ribbon",dotr.Maia.CurrentProject:Name)
           BuildDirectory = substitute("&1/UI/winform/Ribbon",dotr.Maia.CurrentProject:Path).
   
  end constructor.

  /** inject generator code into the template
   * any code manually placed in the initializecomponents method is overwritten
   * this method takes the template, looks for the appropriate places
   * and injects generator code into the template, after it has been read from disk
   * this makes sure that no code is lost if the template is loaded into the visual designer
   * 
   * @param p_Data : code for the template
   * @return longchar : the modified template
   */

  method public override longchar StartBuild(p_Data as longchar):
    def var lv_resx    as longchar no-undo.
    
    def var lv_Ribbon     as char no-undo.
    def var lv_RibbonBase as char no-undo.
    
    /** replace all PlaceHolder phrase with the name of the object
      * Placeholder is used so that we can actually load the template
      * into the visual designer and modify it if required */

    assign lv_RibbonBase = substitute("&1/UI/winform/Ribbon/Base/&2BaseRibbon",dotr.Maia.CurrentProject:Name,ObjectConfig1:ClassName)
           lv_Ribbon     = substitute("&1/UI/winform/Ribbon/&2",dotr.Maia.CurrentProject:Name,ObjectConfig1:CurrentClassName).

    assign p_Data = replace(p_Data,"_Maia-Template.Ribbon",replace(lv_Ribbon,"/","."))
           p_Data = replace(p_Data,"_Maia-Template.BaseRibbon",replace(lv_RibbonBase,"/","."))
           
           p_Data = replace(p_Data,"dotr.Maia.Template.Ribbon",replace(lv_Ribbon,"/","."))
           p_Data = replace(p_Data,"dotr.Maia.Template.BaseRibbon",replace(lv_RibbonBase,"/","."))
           
           p_Data = replace(p_Data," Ribbon ",substitute(" &1 ",ObjectConfig1:CurrentClassName))  
            
    .

    return p_Data.
  end method.
end class.