/*
Copyright (c) 2011, Julian Lyndon-Smith (julian+maia@dotr.com)
http://www.dotr.com
All rights reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

using Progress.Lang.*.
using dotr.Maia.Model.*.

class dotr.Maia.Builder.DBProperties: 
  /** read database table, getting fields and create properties based on those fields 
   * @param p_ObjectConfig : object to build
   */
  
  method public void GetDBProperties(p_ObjectConfig as ObjectConfig ):
    def var lv_index    as int no-undo.
    def var lv_Endindex as int no-undo.
    def var lv_MaxLen   as int no-undo.
    def var lv_i        as int no-undo.
    def var lv_TypeLen  as  int  no-undo extent 10 .
    
    def var lv_Data  as longchar no-undo.
    
    def var lv_Field     as char                no-undo.
    def var lv_Options   as char                no-undo.
    def var lv_Indicator as char case-sensitive no-undo.
    
    def var Property1 as Property no-undo.
    def var Index1    as index    no-undo extent.
    
    /** this method will attempt to find the keyfield (either a unique char value or a unique int value */

    assign p_ObjectConfig:KeyFieldName      = "" 
           p_ObjectConfig:KeyProperty = ""
           p_ObjectConfig:KeyFieldDatatype = ""
           
           lv_Indicator    = ""
           
           lv_TypeLen = 0.
  
    find DICTDB._file where DICTDB._File._File-name eq p_ObjectConfig:TableName no-lock no-error.
  
    if not avail DICTDB._File then return. 

    /** loop through all fields */
    for each DICTDB._Field of DICTDB._File no-lock break by _field._data-type by DICTDB._Field._Field-Name:

      /** each datatype is grouped and split by a blank line 
        * we need to identify the longest property name in each datatype
        * so that we can align the code nicely */ 
      if first-of(DICTDB._Field._data-type) then assign lv_Index = lv_Index + 1.
      
      assign lv_TypeLen[lv_index] = MAX(lv_TypeLen[lv_index],length(DICTDB._Field._Field-name) + 1)
             lv_MaxLen            = MAX(lv_TypeLen[lv_index],lv_MaxLen).
  
      assign Property1 = p_ObjectConfig:GetFieldName(DICTDB._Field._Field-name). /** try to find field options for the generator */
      
      assign Property1:checked = yes.
       
      /** set default options */
      if Property1:FieldName eq "" then  
      do:
        assign Property1:FieldName    = DICTDB._Field._Field-name
               Property1:PropertyName = DICTDB._Field._Field-name
               Property1:DataType     = DICTDB._Field._Data-type
               Property1:AccessMode   = "public"
               Property1:extent       = DICTDB._Field._Extent
               lv_Indicator           = substr(Property1:FieldName,1,1).
  
        /** this is for cases where the field name starts with a letter indicating the data type. 
          * for example cCustomer. we don't want a property called foo:cCustomer, so need to strip
          * the leading chars off 
          *
          * Perhaps this would be an idea to enable this option via a flag or something rather than
          * invoking it blindly ? */
          
        if Property1:PropertyName eq "" and lc(lv_Indicator) eq lv_Indicator then /** assuming that a lower-case letter starting a field name indicates the data type */  
        do:
          case lv_Indicator:
            when "i" or when "c" then assign Property1:PropertyName = substr(Property1:FieldName,2).
            
            otherwise
            do:
              assign lv_Indicator = substr(Property1:FieldName,1,2).
              case lv_Indicator:
                when "de" or when "lo" or when "dt" or when "da" then assign Property1:PropertyName = substr(Property1:FieldName,3).
              end case.
            end. 
             
          end case.
          
        end.
      
        else assign Property1:PropertyName = Property1:FieldName.      

        /** add property to object config */
        p_ObjectConfig:AddProperty(Property1).
      end.
      
      /** if field matches <<table>>GUID then automatically set it as a generate guid field */
      if Property1:FieldName eq substitute("&1GUID",p_ObjectConfig:ClassName) then 
      do:
        assign Property1:GenerateGUID          = yes
               p_ObjectConfig:KeyFieldName     = Property1:FieldName
               p_ObjectConfig:KeyProperty      = Property1:FieldName /* deliberate. Field is the same as property */
               p_ObjectConfig:KeyFieldDatatype = "char".
      end.
      
      /** if field matches <<table>>ID or i<<table>>ID then automatically set it as a next-value field */

      if (Property1:FieldName eq substitute("&1ID",p_ObjectConfig:ClassName) or
          Property1:FieldName eq substitute("i&1ID",p_ObjectConfig:ClassName)) and Property1:DataType eq "integer" then
      do:
        if p_ObjectConfig:KeyFieldName eq "" then assign p_ObjectConfig:KeyFieldName     = Property1:FieldName
                                                         p_ObjectConfig:KeyProperty      = substitute("&1ID",p_ObjectConfig:ClassName)
                                                         p_ObjectConfig:KeyFieldDatatype = "int".
        assign Property1:Sequence = Property1:PropertyName.
      end.
      
    end.

    if p_ObjectConfig:KeyFieldName eq "" then /** we still have no keyfield. try to determine by looking at the indicies. */ 
    do:
      Index1 = BuildIndex(p_ObjectConfig).
        
      /** first look for all single-component unique indicies */
      do lv_i = 1 to extent(Index1):
        if not Index1[lv_i]:IsUnique then next.
        if extent(Index1[lv_i]:Component) gt 1 then next.
        
        Property1 = p_ObjectConfig:GetFieldName(Index1[lv_i]:Component[1]). /** try to find field options for the generator */

        assign p_ObjectConfig:KeyFieldName     = Property1:FieldName
               p_ObjectConfig:KeyProperty      = Property1:PropertyName
               p_ObjectConfig:KeyFieldDatatype = Property1:Datatype.
      end.
    end.

    /** clear out old properties */
    do lv_i = 1 to p_ObjectConfig:NumProperties:
      if p_ObjectConfig:Property[lv_i]:ManualProperty then next.
      if p_ObjectConfig:Property[lv_i]:Checked     then next.
      
      assign p_ObjectConfig:Property[lv_i] = ?.
    end.

    /** clear out old indicies */
    do lv_i = 1 to p_ObjectConfig:NumIndex:
      delete object p_ObjectConfig:index[lv_i] no-error.
      assign p_ObjectConfig:index[lv_i] = ?.
    end.
    finally:
      p_ObjectConfig:Save().        
    end finally.
      
  end method.
    
  method public dotr.Maia.Model.Index extent BuildIndex(p_ObjectConfig as ObjectConfig):
    def var Index1    as index    extent no-undo.
    def var Property1 as property        no-undo.
    
    def var lv_index    as int no-undo.
    def var lv_indexNum as int no-undo.
    
    find DICTDB._file where DICTDB._File._File-name eq p_ObjectConfig:TableName no-lock no-error.
      
    if not avail DICTDB._File then undo, throw new Progress.Lang.AppError(substitute("No Database File found for &1 in DB &2",p_ObjectConfig:TableName,ldbname("DICTDB")),0). 

    extent(Index1) = DICTDB._File._numkey.
    
    for each DICTDB._Index of DICTDB._File no-lock lv_IndexNum = 1 to lv_IndexNum + 1 on error undo , throw :
    
      assign Index1[lv_IndexNum] = new Index(). 
      
      assign Index1[lv_IndexNum]:FindString     = ''
             Index1[lv_IndexNum]:parametersCall = ""
             Index1[lv_IndexNum]:parametersIn   = ""
             Index1[lv_IndexNum]:IsUnique       = DICTDB._Index._Unique.
             
      extent(Index1[lv_IndexNum]:Component) = DICTDB._Index._num-comp.
      
      /** loop through all fields of this index */
      for each DICTDB._index-field of DICTDB._Index no-lock lv_Index = 1 to lv_Index + 1 on error undo, throw :
        find DICTDB._Field where recid(DICTDB._Field) eq DICTDB._index-field._field-recid no-lock.
        
        Property1 = p_ObjectConfig:GetFieldName(DICTDB._Field._Field-name). /* try to find field options for the generator */
        
        assign Index1[lv_IndexNum]:IndexName           = Index1[lv_IndexNum]:IndexName      + Property1:PropertyName
               Index1[lv_IndexNum]:Component[lv_index] = DICTDB._Field._Field-name 
               Index1[lv_IndexNum]:parametersIn        = Index1[lv_IndexNum]:parametersIn   + substitute(", p_&1 as &2",Property1:PropertyName,lc(Property1:dataType))
               Index1[lv_IndexNum]:parametersCall      = Index1[lv_IndexNum]:parametersCall + substitute(", p_&1",Property1:PropertyName)
               Index1[lv_IndexNum]:FindString          = Index1[lv_IndexNum]:FindString     + substitute("&1&2.&3 eq &5 + &4 + &6",
                                                                                                  if Index1[lv_IndexNum]:FindString eq "" then "" else " and ",
                                                                                                  p_ObjectConfig:TableName,
                                                                                                  Property1:FieldName,
                                                                                                  substitute('&1&2&3',
                                                                                                             if Property1:dataType begins "char"
                                                                                                                then "p_"
                                                                                                                else "string(p_",
                                                                                                                Property1:PropertyName,
                                                                                                             if Property1:dataType begins "char"
                                                                                                                then ""
                                                                                                                else ")"),
                                                                                                  if Property1:dataType begins "char" 
                                                                                                     then '~'"'
                                                                                                     else '"',
                                                                                                  if Property1:dataType begins "char" 
                                                                                                     then '"~''
                                                                                                     else '"').
      end.
    
      assign Index1[lv_IndexNum]:IndexName = substitute("FindUsing&1",Index1[lv_IndexNum]:IndexName).
      
      assign Index1[lv_IndexNum]:parametersIn   = trim(Index1[lv_IndexNum]:parametersIn,",")
             Index1[lv_IndexNum]:parametersCall = trim(Index1[lv_IndexNum]:parametersCall,",").

    end.
    
    return Index1.
  end method.
end class.