/*
Copyright (c) 2011, Julian Lyndon-Smith (julian+maia@dotr.com)
http://www.dotr.com
All rights reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

/** Builds the TextEdit control from DevExpress
 * @author Julian Lyndon-Smith (julian+maia@dotr.com) 
 */

class dotr.Maia.Builder.DevExpress.TextEdit inherits dotr.Maia.Builder.BuildUIBase
                                             implements dotr.Maia.Builder.IComponentBuilder:
  /** Create the variable definition
   * @param p_name name of the variable
   * @return char : variable definition
   */
  
  method public char Definition(p_Name as char):
    return substitute("def protected var ui&1 as DevExpress.XtraEditors.TextEdit no-undo.~n",p_Name) +
           substitute("def protected var label&1 as DevExpress.XtraEditors.LabelControl no-undo.~n",p_Name).
  end method.
  
  /** Create the new object definition
   * @param p_name name of the variable
   * @return char : new definition
   */

  method public char New(p_Name as char):
    return substitute("THIS-OBJECT:ui&1 = NEW DevExpress.XtraEditors.TextEdit().~n",p_Name) +
           substitute("THIS-OBJECT:label&1 = NEW DevExpress.XtraEditors.LabelControl().~n",p_Name).
  end method.
  
 /** Create the add control definition
   * @param p_name name of the variable
   * @return char : add control definition
   */
 
  method public char AddControl(p_Name as char):
    return substitute("THIS-OBJECT:groupControl1:Controls:Add(THIS-OBJECT:ui&1).~n",p_Name) +
           substitute("THIS-OBJECT:groupControl1:Controls:Add(THIS-OBJECT:label&1).~n",p_Name).
  end method.
  
 /** Create the properties statements
   * @param p_DrawingOptions options of current layout point
   * @return char : properties definition
   */
    
  method public char Properties(p_DrawingOptions as dotr.Maia.Model.DrawingOptions):
    def var lv_Data as char no-undo.
    
    assign p_DrawingOptions:ComponentTabIndex = p_DrawingOptions:ComponentTabIndex + 1.
    
    assign lv_Data = substitute("/*  */~n"     +
                                "/* ui&1 */~n" +
                                "/*  */~n"     +
                                "THIS-OBJECT:ui&1:Location = NEW System.Drawing.Point(&8, &2).~n" +
                                'THIS-OBJECT:ui&1:Name = "&1".~n' +
                                "THIS-OBJECT:ui&1:Size = NEW System.Drawing.Size(&9, 20).~n" +
                                "THIS-OBJECT:ui&1:TabIndex = &4.~n" +
                                                    
                                "/*  */~n" +
                                "/* label&1 */~n" + 
                                "/*  */~n" + 
                                "THIS-OBJECT:label&1:AutoSizeMode = DevExpress.XtraEditors.LabelAutoSizeMode:Horizontal.~n" +
                                "THIS-OBJECT:label&1:Location = NEW System.Drawing.Point(&7, &3).~n" +
                                'THIS-OBJECT:label&1:Name = "label&1".~n' +
                                "THIS-OBJECT:label&1:Size = NEW System.Drawing.Size(26, 13).~n" +
                                "THIS-OBJECT:label&1:TabIndex = &5.~n" +
                                'THIS-OBJECT:label&1:Text = "&6:".~n' ,
                                                    
                                p_DrawingOptions:ComponentName,
                                p_DrawingOptions:ComponentDrawingPointY,
                                p_DrawingOptions:LabelDrawingPointY,
                                p_DrawingOptions:ComponentTabIndex,
                                p_DrawingOptions:ComponentTabIndex - 1,
                                p_DrawingOptions:ComponentLabel,
                                p_DrawingOptions:LabelDrawingPointX,
                                p_DrawingOptions:ComponentDrawingPointX + 10,
                                p_DrawingOptions:ComponentWidth
                                )
          p_DrawingOptions:ComponentDrawingPointY = p_DrawingOptions:ComponentDrawingPointY + 26.
    return lv_Data.          
  end method.
  

end class.
