/*
 Dot R Limited  CONFIDENTIAL
 Unpublished Copyright (c) 2011 Dot R Limited, All Rights Reserved.

 NOTICE:  All information contained herein is, and remains the property of Dot R Limited. The intellectual and technical concepts contained
 herein are proprietary to Dot R Limited and are protected by trade secret or copyright law.
 Dissemination of this information or reproduction of this material is strictly forbidden unless prior written permission is obtained
 from Dot R Limited.  Access to the source code contained herein is hereby forbidden to anyone except current Dot R Limited employees, managers or contractors who have executed 
 Confidentiality and Non-disclosure agreements explicitly covering such access.

 The copyright notice above does not evidence any actual or intended publication or disclosure  of  this source code, which includes  
 information that is confidential and/or proprietary, and is a trade secret, of  Dot R Limited.   
 
 ANY REPRODUCTION, MODIFICATION, DISTRIBUTION, PUBLIC  PERFORMANCE, OR PUBLIC DISPLAY OF OR THROUGH USE OF THIS SOURCE CODE 
 WITHOUT THE EXPRESS WRITTEN CONSENT OF Dot R Limited IS STRICTLY PROHIBITED, AND IN VIOLATION OF APPLICABLE LAWS AND INTERNATIONAL TREATIES.  
 THE RECEIPT OR POSSESSION OF  THIS SOURCE CODE AND/OR RELATED INFORMATION DOES NOT CONVEY OR IMPLY ANY RIGHTS  
 TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS CONTENTS, OR TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT  MAY DESCRIBE, IN WHOLE OR IN PART.
*/

using Progress.Lang.*.
using dotr.Maia.Stomp.*.
using dotr.Maia.Stomp.Interface.*.

routine-level on error undo, throw.

class dotr.Maia.Stomp.base.StompConnection abstract:
  def var SubscriptionIDCounter as int no-undo.
  
  def public event MessageReceived signature void (p_StompMessage as IStompMessage).
  
  def public property ConnectionID as char init ""          no-undo get . private set .

  def var ProcedureHandler as handle no-undo.

  def var ClassHandler     as IStompHandler no-undo.

  def var SocketHandle  as handle no-undo.
  def var SocketHandler as handle no-undo.
  
  constructor StompConnection(p_StompServer as StompServer):
    ConnectToServer(p_StompServer).
  end constructor.

  constructor StompConnection(p_StompServer as StompServer,p_Handler as handle):
    ConnectToServer(p_StompServer).
    AddHandler(p_Handler).
  end constructor.

  constructor StompConnection(p_StompServer as StompServer,p_Handler as dotr.Maia.Stomp.Interface.IStompHandler):
    ConnectToServer(p_StompServer).
    AddHandler(p_Handler).
  end constructor.
    
  destructor StompConnection():
    MessageReceived:Unsubscribe(ClassHandler:NewStompMessage)       no-error.
    MessageReceived:Unsubscribe(ProcedureHandler,"NewStompMessage") no-error.

    if valid-handle(SocketHandle) then 
    do:
      SocketHandle:disconnect().
      delete object SocketHandle.
    end.

    if valid-handle(SocketHandler) then delete procedure SocketHandler.
  end destructor.
  
  method public void Disconnect():
    sendFrame(substitute("DISCONNECT~n~n",  this-object:ConnectionId)).
  end method.

  method private void AddHandler(p_Handler as handle):
    if valid-object(ClassHandler) then 
    do:
      MessageReceived:Unsubscribe(ClassHandler:NewStompMessage).
    end.
  
    if valid-handle(ProcedureHandler) then 
    do:
      MessageReceived:Unsubscribe(ProcedureHandler,"NewStompMessage").   
    end.
    
    MessageReceived:Subscribe(p_Handler,"NewStompMessage").  
    
    assign ProcedureHandler = p_handler
           ClassHandler     = ?.
  end method.

  method private void AddHandler(p_Handler as IStompHandler):
    if valid-object(ClassHandler) then 
    do:
      MessageReceived:Unsubscribe(ClassHandler:NewStompMessage).
    end.
  
    if valid-handle(ProcedureHandler) then 
    do:
      MessageReceived:Unsubscribe(ProcedureHandler,"NewStompMessage").   
    end.
    
    MessageReceived:Subscribe(p_Handler:NewStompMessage).  
    
    assign ProcedureHandler = ?
           ClassHandler     = p_Handler.
  end method.

  method public void ProcessData(p_Message as longchar):
    def var lv_Data as longchar no-undo.
    
    case string(entry(1, p_Message, "~n")):
      when "CONNECTED" then processConnected(p_Message).
      when "MESSAGE"   then processMessage(p_Message).
    end case.
  end method.
    
  method private void processConnected(p_Message as longchar):
    def var lv_Index as int no-undo.
    assign lv_Index     = index(p_Message,"session:id:")
           ConnectionId = replace(substring(p_message,lv_index),"session:id:","").
  end method.

  method private void processMessage(p_Message as longchar):
    def var lv_Dest   as char no-undo.
    def var lv_Header as char no-undo.
    def var lv_Body   as char no-undo.
    
    def var lv_Index as int no-undo.
    
    assign lv_index  = index(p_Message,"~n~n")
           lv_Body   = trim(substring(p_message,lv_index + 1))
           lv_Header = trim(substring(p_message,1,lv_index))
           lv_Index  = index(p_Message,"destination:")
           lv_Dest   = entry(2,substring(p_message,lv_index,index(p_Message,"~n",lv_index) - lv_index),":").
           lv_Dest   = substr(lv_Dest,index(lv_Dest,"/",2) + 1).
           
    this-object:Received(lv_Dest,lv_Header,lv_Body).
  end method.
  
  method protected void Received(p_Dest as char,p_Header as char, p_Body as char):
  end method.

  method public void NewMessage(p_StompMessage as IStompMessage):
    MessageReceived:Publish(p_StompMessage).
  end method.

  method public void sendframe(input iFrame as longchar):
    def var lMemPtr as memptr no-undo.
    def var lNumBytes as int no-undo.

    lNumBytes = length(iFrame) + 1.  /* add 1 bytes for 0-terminated string */
    
    set-size(lMemPtr) = lNumBytes.
    
    put-string(lMemPtr, 1) = iFrame. 
    
    SocketHandle:write(lMemPtr, 1, lNumBytes) .
    if SocketHandle:bytes-written ne lNumBytes then message "oooop" view-as alert-box. 
    set-size(lMemPtr) = 0.
  end method.

  method private void ConnectToServer(p_StompServer as StompServer):

    if not valid-handle(SocketHandle) then 
    do:
      create socket SocketHandle.
            
      run dotr/Maia/Stomp/base/stompSocketProc.p persistent set SocketHandler(this-object).

      SocketHandle:set-read-response-procedure("tcp_readhandler",SocketHandler).
    end.

    SocketHandle:connect(substitute("-H &1 -S &2", p_StompServer:server, p_StompServer:port)).
    
    SocketHandle:set-socket-option ("SO-RCVBUF","4096").
    SocketHandle:set-socket-option ("TCP-NODELAY","TRUE").
    SocketHandle:set-socket-option ("SO-KEEPALIVE","TRUE").
    
    sendFrame(substitute("CONNECT~naccept-version:1.0,1.1,2.0~nhost:&3~nlogin: &1~npasscode:&2~n~n",  p_StompServer:UserName, p_StompServer:Password,p_StompServer:server)).
  end method.

    
end class.