{&! DefineOption &builder="dotr.Maia.Builder.BuildActiveRecordBase" !}
{&! DefineOption &NoSuffix="yes" !}
{&! Copyright !}

using Progress.Lang.*.
using <<AppPackage>>.helper.*.
using <<AppPackage>>.model.*.
using Progress.Data.*.

routine-level on error undo, throw.

class <<AppPackage>>.Model.Base.<<ClassName>>:
  def buffer <<tablename>> for <<DB>>.<<tablename>>.

  def public property rowid as rowid  get. set. /* must be undo, as any error on creation must be reset to ? */
  def public property Query as handle no-undo get . protected set .     
  
  def protected property buffer     as handle no-undo get . protected set .
  def protected property PrimaryKey as handle no-undo get . protected set .
  
  def public property PrivateData as char no-undo get. set.
  
  def public property Available as logical no-undo
    get():
      return rowid ne ?.
    end get . protected set.
    
  def public property NumRecords as int no-undo
    get():
      return query:num-results.
    end get . protected set.

  {&! DefineProperties !}
  
  constructor <<ClassName>>():
    StartUp().
  end constructor.


  /** Creates the buffer handle and buffer fields 
   */

  method protected void StartUp():
    create query this-object:Query.
    
    assign this-object:buffer = buffer <<TableName>>:handle
           this-object:PrimaryKey = this-object:buffer:buffer-field("<<KeyFieldName>>"):handle.
            
    this-object:Query:add-buffer (this-object:buffer).
  end method.

  /** gets the <<ClassName>> matching the unique ID 
   *  @param p_<<ClassName>>ID : the id to find
   */

  method public <<ModelPackage>>.<<ClassName>> find(p_<<ClassName>>ID as <<KeyFieldDatatype>>):
    return this-object:FindWhere(substitute("<<TableName>>.<<KeyFieldName>> eq '&1'",p_<<ClassName>>ID)).
  end.

  /** stub method to allow for override
   */

  method public void AfterGet():
  end method.

 /** stub method to allow for override
   */

  method public void BeforeGet():
  end method.

 /** stub method to allow for override
  */

  method public void BeforeCreate():
  end method.
  
 /** stub method to allow for override
  */

  method public void AfterCreate():
  end method.

 /** stub method to allow for override
  */

  method public void BeforeUpdate():
  end method.
  
 /** stub method to allow for override
  */

  method public void AfterUpdate():
  end method.
  
 /** stub method to allow for override
  */

  method public void BeforeRemove():
  end method.
  
 /** stub method to allow for override
  */

  method public void AfterRemove():
  end method.

 /** stub method to allow for override
  */
  method public void NotFound():
  end method.

  /** assigns the buffer fields to the object
   *  @return <<ModelPackage>>.<<ClassName>>  
   */

  method protected <<ModelPackage>>.<<ClassName>> get():
    BeforeGet().
    assign this-object:rowid = ?.
        
    if not this-object:buffer:available then
    do: 
      NotFound().
      return cast(this-object,<<ModelPackage>>.<<ClassName>>). 
    end.

    assign {&! ActiveRecordModel_AssignFromBuffer !}
           
    AfterGet().
               
    return cast(this-object,<<ModelPackage>>.<<ClassName>>). 
  end.
  
  /** open a query for all records in the <<Table>> table
   */
  method public <<ModelPackage>>.<<ClassName>> all():
    return this-object:FindWhere("").
  end method.

  /** opens a query for records matching the p_Where clause
   *  @param p_Where : the where statement to use  
   *  @return <<ModelPackage>>.<<ClassName>>  
   */
  
  method public <<ModelPackage>>.<<ClassName>> FindWhere (p_BindingSource as BindingSource,p_Where as char):
    this-object:FindWhere(p_Where).
    p_BindingSource:handle = this-object:query.
    return cast(this-object,<<ModelPackage>>.<<ClassName>>). 
  end method.

  /** opens a query for records matching the p_Where clause
   *  @param p_Where : the where statement to use  
   *  @return <<ModelPackage>>.<<ClassName>>  
   */
  
  method public <<ModelPackage>>.<<ClassName>> FindWhere (p_Where as char):
    this-object:Query:query-prepare(substitute("for each <<TableName>> &1 no-lock",
                                                if trim(p_Where) begins "where " 
                                                   then p_Where
                                                   else if p_Where ne "" then substitute("where &1",p_Where)
                                                   else "")).
    this-object:Query:query-open().
    return this-object:First().                                                        
  end method.

  /** opens a query for records matching the p_Where clause
   *  @param p_Where : the where statement to use  
   *  @return <<ModelPackage>>.<<ClassName>>  
   */
  
  method public <<ModelPackage>>.<<ClassName>> preselect (p_Where as char):
    this-object:Query:query-prepare(substitute("preselect each <<TableName>> &1 no-lock",
                                                if trim(p_Where) begins "where " 
                                                   then p_Where
                                                   else if p_Where ne "" then substitute("where &1",p_Where)
                                                   else "")).
    this-object:Query:query-open().
    return this-object:First().                                                        
  end method.
  
  /** gets the first <<ClassName>> of the query
   *  @return <<ModelPackage>>.<<ClassName>>  
   */

  method public <<ModelPackage>>.<<ClassName>> first():
    this-object:Query:get-first().
    return this-object:Get().
  end method.
    
  /** gets the last <<ClassName>> of the query
   *  @return <<ModelPackage>>.<<ClassName>>  
   */

  method public <<ModelPackage>>.<<ClassName>> last():
    this-object:Query:get-last().
    return this-object:Get().
  end method.

  /** gets the next <<ClassName>> of the query
   *  @return <<ModelPackage>>.<<ClassName>>  
   */

  method public <<ModelPackage>>.<<ClassName>> next():
    this-object:Query:get-next().
    return this-object:Get().
  end method.

  /** gets the prev <<ClassName>> of the query
   *  @return <<ModelPackage>>.<<ClassName>>  
   */

  method public <<ModelPackage>>.<<ClassName>> prev():
    this-object:Query:get-prev().
    return this-object:Get().
  end method.

  /** saves the object attributes to the database
   *  @return <<ModelPackage>>.<<ClassName>>  
   */
  
  method public <<ModelPackage>>.<<ClassName>> save():
    WriteRecord(if this-object:rowid eq ? then "CREATE" else "UPDATE").
    return cast(this-object,<<ModelPackage>>.<<ClassName>>). 
  end method.

  /** removes the object from the database
   */
 
  method public void Remove():
    if not this-object:available then return.
    WriteRecord("Remove").
    return. 
  end method. 

  /** writes the approriate changes to the database
   *  @param p_Mode : create (creates a new <<table>>
   *                : update (saves the changes to the database)
   *                : remove (removes the record from the database)
   */
    
  method protected final void WriteRecord(p_Mode as char):
    def var lv_new as logical no-undo.
    
    def buffer <<TableName>> for <<DB>>.<<TableName>>.
    
    assign lv_new = this-object:rowid eq ?.
    
    /** try to lock the record, give up after 5 seconds */
    repeat while true stop-after 5 transaction on error undo, throw
                                               on stop undo, leave:

      if p_mode eq "CREATE" then 
      do:
        BeforeCreate(). /** override this method if you want to do stuff before the create statement */ 

        create <<TableName>>.
        {&! NextValue !}
        {&! GenerateGUID !}
        {&! CreateAssign !}
        
        assign this-object:rowid = rowid(<<tablename>>).
               
        AfterCreate().  /** override this method if you want to do stuff after the create statement */
      end.
      
      else 
      do:
        find <<TableName>> where <<TableName>>.<<KeyFieldName>> eq this-object:<<KeyProperty>> exclusive no-wait no-error.
   
        if locked(<<TableName>>) then next.
        if not avail <<TableName>> then undo, throw new AppError(substitute("Could not find <<TableName>> [&1]",this-object:<<KeyProperty>>),0).
      end.
      
      case p_Mode:
        when "Update" or when "Create" then
        do:
          BeforeUpdate().  /** override this method if you want to do stuff before the update statement */
          assign {&! ActiveRecordModel_AssignToBuffer !}
          AfterUpdate().  /** override this method if you want to do stuff after the update statement */
        end.
        
        when "Remove" then
        do:
          BeforeRemove().  /** override this method if you want to do stuff before the remove statement */
          delete <<TableName>>.
          assign this-object:rowid = ?.

          {&! ActiveRecordDelete !}

          AfterRemove().  /** override this method if you want to do stuff after the remove statement */
        end.
        
      end case.
      
      dotr.Maia.Stomp.Helper.SendMessage:ToTopic("<<AppPackage>>.info.db.update.<<ClassName>>",substitute("action:&1,guid:&2",p_mode,this-object:<<KeyProperty>>)).
      
      release <<TableName>>.
      
      leave.
      
      catch e as Progress.Lang.Error :
        case e:GetMessageNum(1):
          when 132 then undo, throw new AppError(e:getMessage(1),132).
          otherwise     undo, throw new AppError(e:GetMessage(1),0).
        end case.
      end catch.
    end.
    
    return.
    
  end method.    

  {&! ActiveRecordModel_fkDeleted !}
  {&! ActiveRecordModel_fkAssign !}
end class.
