/*
Copyright (c) 2011, Julian Lyndon-Smith (julian+maia@dotr.com)
http://www.dotr.com
All rights reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

using Progress.Lang.*.

routine-level on error undo, throw.

/** skin manipulation class */
class dotr.Maia.UI.Library.Skin:
  
  /** declare static property to enforce a singleton
   *
   * this mechanism allows us to define a singleton instance without
   * having to define the properties and methods as statics. This allows
   * us to unload the instance if required
   */
   
  def public static property Instance as class dotr.Maia.UI.Library.Skin no-undo 
    get():
      if not valid-object(Instance) then Instance = new dotr.Maia.UI.Library.Skin().
      return Instance.
    end get. private set .

  /** list of available skins */
  def public property SkinList    as char no-undo get . private set. 
  
  /** the current skin selected by the user */
  def public property CurrentSkin as char no-undo get . 
    set(p_NewSkin as char):
        
      /** the skin name parameter sometimes is not the same as the actual skin (!) */
      case p_NewSkin:
          when "Xmas (Blue)" then assign p_NewSkin = "Xmas 2008 Blue".
          when "Summer"      then assign p_NewSkin = "Summer 2008".
      end case.
    
      assign CurrentSkin = p_NewSkin.
      dotr.Maia.UI.Library.Skin:SkinChanged:Publish(p_newSkin).
    end set. 
  
  /** declare an event to signify to the session that the skin as changed */
  def public static event SkinChanged signature void(p_NewSkin as char).
  
  /** private constructor means that an instance of a class can only be created by itself. This can only
   *  happen through the instance property */
  constructor private Skin():
    def var lv_NumSkins as int no-undo.
    def var lv_i        as int no-undo.

    DevExpress.Skins.SkinManager:EnableFormSkins().
    DevExpress.UserSkins.BonusSkins:Register().
    DevExpress.UserSkins.OfficeSkins:Register().
    
    assign lv_numSkins = DevExpress.Skins.SkinManager:Default:Skins:Count - 1.
    
    /** 0-based index */
    do lv_i = 0 to lv_numSkins:
      assign SkinList = SkinList + "," + DevExpress.Skins.SkinManager:Default:Skins[lv_i]:SkinName. 
    end.
    
    assign SkinList = trim(SkinList,",").
  
  end constructor.
  
end class.