
/*
Copyright (C) 2011 by Julian Lyndon-Smith (julian+mit@dotr.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS for A PARTICULAR PURPOSE and NONINFRINGEMENT. IN no EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE for ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR otherwise, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/


using Progress.Lang.*.
using sports.helper.*.
using sports.model.*.
using Progress.Data.*.

routine-level on error undo, throw.

class sports.Model.Base.Salesrep:
  def buffer Salesrep for sports.Salesrep.

  def public property rowid as rowid  get. set. /* must be undo, as any error on creation must be reset to ? */
  def public property Query as handle no-undo get . protected set .     
  
  def protected property buffer     as handle no-undo get . protected set .
  def protected property PrimaryKey as handle no-undo get . protected set .
  
  def public property PrivateData as char no-undo get. set.
  
  def public property Available as logical no-undo
    get():
      return rowid ne ?.
    end get . protected set.
    
  def public property NumRecords as int no-undo
    get():
      return query:num-results.
    end get . protected set.

  def public property Region   as character no-undo  get . set . 
  def public property RepName  as character no-undo  get . set . 
  def public property SalesRep as character no-undo  get . set . 
  
  def public property MonthQuota as integer no-undo extent 12 get . set . 
  
  
  constructor Salesrep():
    StartUp().
  end constructor.


  /** Creates the buffer handle and buffer fields 
   */

  method protected void StartUp():
    create query this-object:Query.
    
    assign this-object:buffer = buffer Salesrep:handle
           this-object:PrimaryKey = this-object:buffer:buffer-field("Sales-Rep"):handle.
            
    this-object:Query:add-buffer (this-object:buffer).
  end method.

  /** gets the Salesrep matching the unique ID 
   *  @param p_SalesrepID : the id to find
   */

  method public sports.Model.Salesrep find(p_SalesrepID as Character):
    return this-object:FindWhere(substitute("Salesrep.Sales-Rep eq '&1'",p_SalesrepID)).
  end.

  /** stub method to allow for override
   */

  method public void AfterGet():
  end method.

 /** stub method to allow for override
   */

  method public void BeforeGet():
  end method.

 /** stub method to allow for override
  */

  method public void BeforeCreate():
  end method.
  
 /** stub method to allow for override
  */

  method public void AfterCreate():
  end method.

 /** stub method to allow for override
  */

  method public void BeforeUpdate():
  end method.
  
 /** stub method to allow for override
  */

  method public void AfterUpdate():
  end method.
  
 /** stub method to allow for override
  */

  method public void BeforeRemove():
  end method.
  
 /** stub method to allow for override
  */

  method public void AfterRemove():
  end method.

 /** stub method to allow for override
  */
  method public void NotFound():
  end method.

  /** assigns the buffer fields to the object
   *  @return sports.Model.Salesrep  
   */

  method protected sports.Model.Salesrep get():
    BeforeGet().
    assign this-object:rowid = ?.
        
    if not this-object:buffer:available then
    do: 
      NotFound().
      return cast(this-object,sports.Model.Salesrep). 
    end.

    assign this-object:Region     = buffer::Region
           this-object:RepName    = buffer::Rep-Name
           this-object:SalesRep   = buffer::Sales-Rep
           this-object:MonthQuota = buffer::Month-Quota
           this-object:rowid      = buffer:rowid.
           
    AfterGet().
               
    return cast(this-object,sports.Model.Salesrep). 
  end.
  
  /** open a query for all records in the <<Table>> table
   */
  method public sports.Model.Salesrep all():
    return this-object:FindWhere("").
  end method.

  /** opens a query for records matching the p_Where clause
   *  @param p_Where : the where statement to use  
   *  @return sports.Model.Salesrep  
   */
  
  method public sports.Model.Salesrep FindWhere (p_BindingSource as BindingSource,p_Where as char):
    this-object:FindWhere(p_Where).
    p_BindingSource:handle = this-object:query.
    return cast(this-object,sports.Model.Salesrep). 
  end method.

  /** opens a query for records matching the p_Where clause
   *  @param p_Where : the where statement to use  
   *  @return sports.Model.Salesrep  
   */
  
  method public sports.Model.Salesrep FindWhere (p_Where as char):
    this-object:Query:query-prepare(substitute("for each Salesrep &1 no-lock",
                                                if trim(p_Where) begins "where " 
                                                   then p_Where
                                                   else if p_Where ne "" then substitute("where &1",p_Where)
                                                   else "")).
    this-object:Query:query-open().
    return this-object:First().                                                        
  end method.

  /** opens a query for records matching the p_Where clause
   *  @param p_Where : the where statement to use  
   *  @return sports.Model.Salesrep  
   */
  
  method public sports.Model.Salesrep preselect (p_Where as char):
    this-object:Query:query-prepare(substitute("preselect each Salesrep &1 no-lock",
                                                if trim(p_Where) begins "where " 
                                                   then p_Where
                                                   else if p_Where ne "" then substitute("where &1",p_Where)
                                                   else "")).
    this-object:Query:query-open().
    return this-object:First().                                                        
  end method.
  
  /** gets the first Salesrep of the query
   *  @return sports.Model.Salesrep  
   */

  method public sports.Model.Salesrep first():
    this-object:Query:get-first().
    return this-object:Get().
  end method.
    
  /** gets the last Salesrep of the query
   *  @return sports.Model.Salesrep  
   */

  method public sports.Model.Salesrep last():
    this-object:Query:get-last().
    return this-object:Get().
  end method.

  /** gets the next Salesrep of the query
   *  @return sports.Model.Salesrep  
   */

  method public sports.Model.Salesrep next():
    this-object:Query:get-next().
    return this-object:Get().
  end method.

  /** gets the prev Salesrep of the query
   *  @return sports.Model.Salesrep  
   */

  method public sports.Model.Salesrep prev():
    this-object:Query:get-prev().
    return this-object:Get().
  end method.

  /** saves the object attributes to the database
   *  @return sports.Model.Salesrep  
   */
  
  method public sports.Model.Salesrep save():
    WriteRecord(if this-object:rowid eq ? then "CREATE" else "UPDATE").
    return cast(this-object,sports.Model.Salesrep). 
  end method.

  /** removes the object from the database
   */
 
  method public void Remove():
    if not this-object:available then return.
    WriteRecord("Remove").
    return. 
  end method. 

  /** writes the approriate changes to the database
   *  @param p_Mode : create (creates a new <<table>>
   *                : update (saves the changes to the database)
   *                : remove (removes the record from the database)
   */
    
  method protected final void WriteRecord(p_Mode as char):
    def var lv_new as logical no-undo.
    
    def buffer Salesrep for sports.Salesrep.
    
    assign lv_new = this-object:rowid eq ?.
    
    /** try to lock the record, give up after 5 seconds */
    repeat while true stop-after 5 transaction on error undo, throw
                                               on stop undo, leave:

      if p_mode eq "CREATE" then 
      do:
        BeforeCreate(). /** override this method if you want to do stuff before the create statement */ 

        create Salesrep.
        
        assign this-object:rowid = rowid(Salesrep).
               
        AfterCreate().  /** override this method if you want to do stuff after the create statement */
      end.
      
      else 
      do:
        find Salesrep where Salesrep.Sales-Rep eq this-object:SalesRep exclusive no-wait no-error.
   
        if locked(Salesrep) then next.
        if not avail Salesrep then undo, throw new AppError(substitute("Could not find Salesrep [&1]",this-object:SalesRep),0).
      end.
      
      case p_Mode:
        when "Update" or when "Create" then
        do:
          BeforeUpdate().  /** override this method if you want to do stuff before the update statement */
          assign Salesrep.Region          = this-object:Region
                 Salesrep.Rep-Name        = this-object:RepName
                 Salesrep.Sales-Rep       = this-object:SalesRep
                 Salesrep.Month-Quota[1]     = this-object:MonthQuota[1]
                 Salesrep.Month-Quota[2]     = this-object:MonthQuota[2]
                 Salesrep.Month-Quota[3]     = this-object:MonthQuota[3]
                 Salesrep.Month-Quota[4]     = this-object:MonthQuota[4]
                 Salesrep.Month-Quota[5]     = this-object:MonthQuota[5]
                 Salesrep.Month-Quota[6]     = this-object:MonthQuota[6]
                 Salesrep.Month-Quota[7]     = this-object:MonthQuota[7]
                 Salesrep.Month-Quota[8]     = this-object:MonthQuota[8]
                 Salesrep.Month-Quota[9]     = this-object:MonthQuota[9]
                 Salesrep.Month-Quota[10]     = this-object:MonthQuota[10]
                 Salesrep.Month-Quota[11]     = this-object:MonthQuota[11]
                 Salesrep.Month-Quota[12]     = this-object:MonthQuota[12].
          AfterUpdate().  /** override this method if you want to do stuff after the update statement */
        end.
        
        when "Remove" then
        do:
          BeforeRemove().  /** override this method if you want to do stuff before the remove statement */
          delete Salesrep.
          assign this-object:rowid = ?.


          AfterRemove().  /** override this method if you want to do stuff after the remove statement */
        end.
        
      end case.
      
      dotr.Maia.Stomp.Helper.SendMessage:ToTopic("sports.info.db.update.Salesrep",substitute("action:&1,guid:&2",p_mode,this-object:SalesRep)).
      
      release Salesrep.
      
      leave.
      
      catch e as Progress.Lang.Error :
        case e:GetMessageNum(1):
          when 132 then undo, throw new AppError(e:getMessage(1),132).
          otherwise     undo, throw new AppError(e:GetMessage(1),0).
        end case.
      end catch.
    end.
    
    return.
    
  end method.    

end class.
@BuildInfo(Source="./sports/_Maia-Config/Salesrep/_Config.xml").
@BuildInfo(Date="30/08/2011 08:58:04.780+01:00").